use kameo::message::Message;

use crate::{
    s,
    server::actor::messages::{RepoUpdate, ServerUpdate},
    xui::state::{CIStatus, UiFocus},
};

use super::{
    state::{RepoKey, RepoStateStatus},
    Xui,
};

impl Message<ServerUpdate> for Xui {
    type Reply = std::io::Result<()>;

    #[allow(clippy::too_many_lines)]
    async fn handle(
        &mut self,
        msg: ServerUpdate,
        _ctx: kameo::message::Context<'_, Self, Self::Reply>,
    ) -> Self::Reply {
        tracing::info!(?msg);
        match msg {
            ServerUpdate::AppConfigLoaded { app_config } => {
                self.state.data.set_app_config(&app_config);
                if self.state.ui.table_state.selected().is_none()
                    && !self.state.data.repos().is_empty()
                {
                    self.state.ui.table_state.select(Some(0));
                }
            }
            ServerUpdate::RepoUpdate {
                forge_alias,
                repo_alias,
                repo_update,
            } => {
                let repo_key = RepoKey::new(forge_alias, repo_alias);
                if let Some(repo_state) = self.state.data.get_repo_state(&repo_key).cloned() {
                    let updated_repo_state = match repo_update {
                        RepoUpdate::UpdateMessageToken(message_token) => {
                            repo_state.with_message_token(message_token)
                        }
                        RepoUpdate::Branches { branches } => {
                            repo_state.with_repo_branches(branches)
                        }
                        RepoUpdate::Log { log } => repo_state.with_log(log),
                        RepoUpdate::ValidateRepo => {
                            repo_state.with_status(RepoStateStatus::Validate)
                        }
                        RepoUpdate::Okay { main, next, dev } => repo_state
                            .with_status(RepoStateStatus::Okay)
                            .with_ci_status(CIStatus::Okay)
                            .with_main(main)
                            .with_next(next)
                            .with_dev(dev),
                        RepoUpdate::Alert { alert } => {
                            repo_state.with_status(RepoStateStatus::Alert { alert })
                        }
                        RepoUpdate::CheckingCI => repo_state
                            .with_status(RepoStateStatus::CheckingCI)
                            .with_ci_status(CIStatus::Checking),
                        RepoUpdate::AdvancingNext { commit, force } => {
                            repo_state.with_status(RepoStateStatus::AdvancingNext { commit, force })
                        }
                        RepoUpdate::AdvancingMain { commit } => {
                            repo_state.with_status(RepoStateStatus::AdvancingMain { commit })
                        }
                        RepoUpdate::Opening { repo_actor } => repo_state
                            .with_repo_actor(repo_actor)
                            .with_status(RepoStateStatus::Opening),
                        RepoUpdate::LoadingConfigFromRepo => {
                            repo_state.with_status(RepoStateStatus::LoadingConfigFromRepo)
                        }
                        RepoUpdate::ReceiveCIStatus { status } => repo_state
                            .with_ci_status(CIStatus::Received(status.clone()))
                            .with_status(RepoStateStatus::ReceiveCIStatus { status }),
                        RepoUpdate::ReceiveRepoConfig { repo_config } => repo_state
                            .with_status(RepoStateStatus::ReceiveRepoConfig)
                            .with_repo_branches(repo_config.branches().clone()),
                        RepoUpdate::RegisteringWebhook => {
                            repo_state.with_status(RepoStateStatus::RegisteringWebhook)
                        }
                        RepoUpdate::UnregisteringWebhook => {
                            repo_state.with_status(RepoStateStatus::UnregisteringWebhook)
                        }
                        RepoUpdate::WebhookReceived { push } => {
                            repo_state.with_status(RepoStateStatus::WebhookReceived { push })
                        }
                        RepoUpdate::RegisteredWebhook => {
                            repo_state.with_status(RepoStateStatus::RegisteredWebhook)
                        }
                        RepoUpdate::Opened => repo_state.with_status(RepoStateStatus::Opened),
                        RepoUpdate::NextUpdated => {
                            repo_state.with_status(RepoStateStatus::NextUpdated)
                        }
                        RepoUpdate::MainUpdated => {
                            repo_state.with_status(RepoStateStatus::MainUpdated)
                        }
                        RepoUpdate::NotOpen => repo_state.with_status(RepoStateStatus::Error {
                            error: s!("repo not open"),
                        }),
                        RepoUpdate::NoConfig => repo_state.with_status(RepoStateStatus::Error {
                            error: s!("repo has no config"),
                        }),
                        RepoUpdate::UserIntervention(notification) => repo_state
                            .with_status(RepoStateStatus::UserIntervention { notification }),
                        RepoUpdate::Error(message) => {
                            repo_state.with_status(RepoStateStatus::Error { error: message })
                        }
                        RepoUpdate::RebaseError(err) => {
                            self.state.ui.focus = UiFocus::RebaseError(err);
                            repo_state
                        }
                        RepoUpdate::ProgressMessage(message) => {
                            self.state.ui.focus = UiFocus::Progress(message);
                            repo_state
                        }
                        RepoUpdate::RebaseCompleted => {
                            self.state.ui.focus = UiFocus::RebaseCompleted;
                            repo_state
                        }
                    };
                    self.state.data.set_repo_state(repo_key, updated_repo_state);
                }
            }
        }
        self.draw()
    }
}
