use kameo::actor::ActorRef;
use ratatui::style::{Style, Stylize};

use crate::{
    core::{
        git::{self, forge::commit::Status, push::Force, Commit},
        webhook::Push,
        RepoBranches,
    },
    repo::{messages::MessageToken, RepoActor},
};

#[derive(Clone, Debug, Default, derive_with::With)]
pub struct RepoState {
    pub repo_actor: Option<ActorRef<RepoActor>>,
    pub message_token: MessageToken,
    pub repo_branches: RepoBranches,
    pub log: git::graph::Log,
    pub status: RepoStateStatus,
    pub main: Option<Commit>,
    pub next: Option<Commit>,
    pub dev: Option<Commit>,
    pub ci_status: Option<CIStatus>,
}

#[derive(
    Clone, Debug, derive_more::Display, Default, PartialEq, Eq, serde::Serialize, serde::Deserialize,
)]
pub enum CIStatus {
    #[default]
    Unknown,
    Okay,
    Checking,
    Received(Status),
}

#[derive(
    Clone, Debug, derive_more::Display, Default, PartialEq, Eq, serde::Serialize, serde::Deserialize,
)]
pub enum RepoStateStatus {
    #[default]
    Unknown,
    Validate,
    #[display("! {alert}")]
    Alert {
        alert: String,
    },
    #[display("Checking CI...")]
    CheckingCI,
    #[display("Advancing Next")]
    AdvancingNext {
        commit: Commit,
        force: Force,
    },
    #[display("Advancing Main")]
    AdvancingMain {
        commit: Commit,
    },
    Opening,
    #[display("Loading Repo Config")]
    LoadingConfigFromRepo,
    #[display("CI status: {status}")]
    ReceiveCIStatus {
        status: Status,
    },
    #[display("Received Repo Config")]
    ReceiveRepoConfig,
    #[display("Registering Webhook")]
    RegisteringWebhook,
    #[display("Unregistering Webhook")]
    UnregisteringWebhook,
    #[display("Webhook Received: {}", push.branch)]
    WebhookReceived {
        push: Push,
    },
    #[display("Registered Webhook")]
    RegisteredWebhook,
    Opened,
    #[display("Next Updated")]
    NextUpdated,
    #[display("Main Updated")]
    MainUpdated,
    #[display("! {error}")]
    Error {
        error: String,
    },
    Okay,
    UserIntervention {
        notification: crate::core::git::UserNotification,
    },
}
impl RepoStateStatus {
    pub fn style(&self) -> Style {
        let red = Style::default().red();
        let amber = Style::default().yellow();
        let green = Style::default().green();

        match self {
            Self::Unknown | Self::RegisteredWebhook | Self::Opened | Self::Okay => green,
            Self::Alert { .. } | Self::Error { .. } | Self::UserIntervention { .. } => red,
            Self::Validate
            | Self::Opening
            | Self::CheckingCI
            | Self::AdvancingNext { .. }
            | Self::AdvancingMain { .. }
            | Self::LoadingConfigFromRepo
            | Self::ReceiveRepoConfig
            | Self::RegisteringWebhook
            | Self::UnregisteringWebhook
            | Self::WebhookReceived { .. }
            | Self::MainUpdated
            | Self::NextUpdated => amber,
            Self::ReceiveCIStatus { status } => match status {
                Status::Fail | Status::Error(_) => red,
                Status::Pending => amber,
                Status::Pass => green,
            },
        }
    }
}
#[cfg(test)]
mod test {
    use super::RepoStateStatus;

    #[test]
    fn display() {
        let status = RepoStateStatus::Unknown;

        let _str: String = status.to_string();
    }
}
