use std::string::ToString;

use ratatui::{
    layout::{Alignment, Constraint, Direction, Layout, Rect},
    style::{Style, Stylize as _},
    symbols::border,
    text::{Line, Span},
    widgets::{Block, StatefulWidget, Widget as _},
};

use crate::xui::{
    state::{DataState, RepoKey, RepoState, UiFocus, UiState},
    widgets::Popup,
};

use super::{RepoDetails, RepoTable, RepoTableItem};

impl StatefulWidget for &DataState {
    type State = UiState;

    /// ┏━━━━━━━━━━━━ Git-Next v2025.2.1 ━━━━━━━━━━━━━━━━━┓
    /// ┃ Forge/Repo  Status      ┏━ Log; jo/git-next ━━━┓┃
    /// ┃                         ┃* (dev) foo           ┃┃
    /// ┃ jo/skip     dev-on-main ┃* (next) testing      ┃┃
    /// ┃>jo/git-next CI Running  ┃* (main) fixed        ┃┃
    /// ┃ jo/kxio     CI Failed   ┃                      ┃┃
    /// ┃                         ┃                      ┃┃
    /// ┃                         ┗━━━━━━━━━━━━━━━━━━━━━━┛┃
    /// ┗ \[q]uit [/] filter (jo) ━━━━━━━━━━━━━━━━━━━━━━━━┛
    #[allow(clippy::too_many_lines)]
    fn render(
        self,
        area: ratatui::prelude::Rect,
        buf: &mut ratatui::prelude::Buffer,
        ui_state: &mut Self::State,
    ) {
        let app_title = Line::from(format!(" Git-Next v{} ", clap::crate_version!()).bold());
        let key_hints = key_hints(ui_state);
        let block = Block::bordered()
            .title_top(app_title.alignment(Alignment::Center))
            .title_bottom(key_hints.alignment(Alignment::Left))
            .border_set(border::THICK);
        let interior = block.inner(area);
        block.render(area, buf);
        let items = self
            .repos()
            .into_iter()
            .map(
                |(
                    RepoKey {
                        forge_alias,
                        repo_alias,
                    },
                    repo_state,
                )| {
                    (
                        format!("{forge_alias}/{repo_alias}"),
                        repo_state.status.to_string(),
                        repo_state.status.style(),
                    )
                },
            )
            .filter(|(forge_repo, _, _)| forge_repo.contains(&ui_state.filter))
            .map(|(forge_repo, status, style)| RepoTableItem {
                forge_repo,
                status,
                style,
            })
            .collect::<Vec<_>>();

        #[allow(clippy::cast_possible_truncation)]
        let forge_repo_width: u16 = items
            .iter()
            .map(|RepoTableItem { forge_repo, .. }| forge_repo.len().clamp(11, 20) as u16)
            .max()
            .unwrap_or(11);
        let status_width = 20;
        let padding = 4; // indicator + pre, mid and post padding
        let repo_table_width = forge_repo_width + status_width + padding;

        let [table_area, repo_area] = Layout::default()
            .direction(Direction::Horizontal)
            .constraints([Constraint::Length(repo_table_width), Constraint::Fill(1)])
            .areas(interior);

        let table = RepoTable { items };
        table.render(table_area, buf, &mut ui_state.table_state);

        if let Some(selection) = ui_state.table_state.selected() {
            if let Some((
                RepoKey {
                    forge_alias,
                    repo_alias,
                },
                RepoState {
                    repo_branches,
                    log,
                    status,
                    ci_status,
                    ..
                },
            )) = self.repos().get(selection)
            {
                let widget = RepoDetails {
                    forge_alias: forge_alias.clone(),
                    repo_alias: repo_alias.clone(),
                    repo_branches: repo_branches.clone(),
                    log: log.clone(),
                    status: status.clone(),
                    ci_state: ci_status.clone(),
                };
                widget.render(repo_area, buf);
                match &ui_state.focus {
                    UiFocus::DevRebaseConfirm => {
                        let popup = Popup::default()
                            .title(" Rebase dev on main? ")
                            .content("Are you sure?")
                            .title_bottom("[Y]es, or [N]o")
                            .style(Style::default())
                            .border_style(Style::default())
                            .title_style(Style::default());
                        let popup_area = centered_rect(60, 20, area);
                        popup.render(popup_area, buf);
                    }
                    UiFocus::Progress(message) => {
                        let popup = Popup::default()
                            .title(" Progress ")
                            .content(message.clone())
                            .title_bottom(" press any key to continue in background ")
                            .style(Style::default())
                            .border_style(Style::default())
                            .title_style(Style::default());
                        let popup_area = centered_rect(60, 20, area);
                        popup.render(popup_area, buf);
                    }
                    UiFocus::RebaseError(error) => {
                        let popup = Popup::default()
                            .title(" Rebase Error ")
                            .content(error.clone())
                            .title_bottom(" press any key to continue ")
                            .style(Style::default())
                            .border_style(Style::default())
                            .title_style(Style::default().red());
                        let popup_area = centered_rect(60, 20, area);
                        popup.render(popup_area, buf);
                    }
                    UiFocus::RebaseCompleted => {
                        let popup = Popup::default()
                            .title(" Rebase Complete ")
                            .content("dev branch has been rebased onto main and pushed")
                            .title_bottom(" press any key to continue ")
                            .style(Style::default())
                            .border_style(Style::default())
                            .title_style(Style::default().red());
                        let popup_area = centered_rect(60, 20, area);
                        popup.render(popup_area, buf);
                    }
                    UiFocus::RepoList | UiFocus::FilterInput => (),
                }
            }
        }
    }
}

/// helper function to create a centered rect using up certain percentage of the available rect `r`
fn centered_rect(percent_x: u16, percent_y: u16, r: Rect) -> Rect {
    let popup_layout = Layout::default()
        .direction(Direction::Vertical)
        .constraints([
            Constraint::Percentage((100 - percent_y) / 2),
            Constraint::Percentage(percent_y),
            Constraint::Percentage((100 - percent_y) / 2),
        ])
        .split(r);

    Layout::default()
        .direction(Direction::Horizontal)
        .constraints([
            Constraint::Percentage((100 - percent_x) / 2),
            Constraint::Percentage(percent_x),
            Constraint::Percentage((100 - percent_x) / 2),
        ])
        .split(popup_layout[1])[1]
}

fn key_hints(ui_state: &UiState) -> Line<'_> {
    let mut hints: Vec<Span<'_>> = vec![Span::from(" ")];
    let filter_hint = format!("({})", ui_state.filter);
    match &ui_state.focus {
        UiFocus::RepoList => {
            hints.extend(vec!["[".into(), "q".red(), "]uit ".into()]);
            hints.extend(vec!["[".into(), "/".red(), "] filter ".into()]);
            if !ui_state.filter.is_empty() {
                hints.push(filter_hint.into());
            }
        }
        UiFocus::FilterInput => {
            hints.push("filter: ".into());
            hints.push(ui_state.filter.clone().into());
            hints.extend(vec![" [".into(), "esc".red(), "] clear ".into()]);
            hints.extend(vec![
                "[".into(),
                "tab".red(),
                "/".into(),
                "enter".red(),
                "] finish ".into(),
            ]);
        }
        UiFocus::DevRebaseConfirm => {
            hints.extend(vec!["[".into(), "y".red(), "]es".into()]);
            hints.extend(vec!["[".into(), "n".red(), "]o".into()]);
        }
        UiFocus::RebaseCompleted | UiFocus::RebaseError(_) | UiFocus::Progress(_) => {}
    };
    Line::from(hints)
}
