use ratatui::{
    layout::{Alignment, Constraint, Layout},
    prelude::{Buffer, Rect},
    text::Line,
    widgets::{Block, Borders, Widget},
};

use crate::{
    core::{
        git::{graph::Log, UserNotification},
        ForgeAlias, RepoAlias, RepoBranches,
    },
    xui::state::{CIStatus, RepoStateStatus},
};

use super::{CommitLog, UserAlert};

#[derive(Clone, Debug, PartialEq, Eq, serde::Serialize, serde::Deserialize)]
pub struct RepoDetails {
    pub forge_alias: ForgeAlias,
    pub repo_alias: RepoAlias,
    pub(crate) repo_branches: RepoBranches,
    pub(crate) log: Log,
    pub(crate) ci_state: Option<CIStatus>,
    pub(crate) status: RepoStateStatus,
}

impl Widget for RepoDetails {
    fn render(self, area: Rect, buf: &mut Buffer)
    where
        Self: Sized,
    {
        let block = Block::default()
            .title(format!(" {}/{} ", self.forge_alias, self.repo_alias))
            .title_bottom(key_hints(&self.status).alignment(Alignment::Left))
            .borders(Borders::ALL);
        let interior = block.inner(area);
        block.render(area, buf);
        let [notification_area, log_area, branches_area] = Layout::default()
            .direction(ratatui::layout::Direction::Vertical)
            .constraints([
                Constraint::Length(1), // user notification
                Constraint::Fill(1),   // Git Log
                Constraint::Length(1), // repo_branches
            ])
            .areas(interior);

        if let RepoStateStatus::UserIntervention { notification } = &self.status {
            UserAlert { notification }.render(notification_area, buf);
        }

        CommitLog { log: &self.log }.render(log_area, buf);

        Line::from(format!(
            " ({} -> {} -> {}) ",
            self.repo_branches.dev(),
            self.repo_branches.next(),
            self.repo_branches.main()
        ))
        .alignment(Alignment::Right)
        .render(branches_area, buf);
    }
}

fn key_hints(status: &RepoStateStatus) -> Line<'_> {
    if let RepoStateStatus::UserIntervention { notification } = status {
        Line::from(match notification {
            UserNotification::CICheckFailed { .. }
            | UserNotification::RepoConfigLoadFailure { .. }
            | UserNotification::WebhookRegistration { .. } => "[r]etry",
            UserNotification::DevNotBasedOnMain { .. } => "[r]ebase",
        })
    } else {
        Line::from("")
    }
}
